﻿using System;

namespace ExcelDummy
{
	public class ExcelApp : IDisposable
	{
		public ExcelApp()
		{
			Console.WriteLine( "ExcelApp Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "ExcelApp Dispose" );
		}

		public BookCollection Books
		{
			get { return new BookCollection(); }
		}
	}

	public class BookCollection : IDisposable
	{
		public BookCollection()
		{
			Console.WriteLine( "BookCollection Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "BookCollection Dispose" );
		}

		public Book Open( string book_path )
		{
			return new Book( book_path );
		}
	}

	public class Book : IDisposable
	{
		public Book( string book_path )
		{
			if( string.IsNullOrEmpty( book_path ) )
			{
				throw new ArgumentNullException();
			}

			Path = book_path;
			Console.WriteLine( "Book(" + Path + ") Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "Book(" + Path + ") Dispose" );
		}

		public SheetCollection Sheets
		{
			get { return new SheetCollection(); }
		}

		public string Path { get; set; }
	}

	public class SheetCollection : IDisposable
	{
		public SheetCollection()
		{
			Console.WriteLine( "SheetCollection Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "SheetCollection Dispose" );
		}

		public Sheet this[ int index ]
		{
			get
			{
				if( index <= 0 )
				{
					throw new ArgumentException();
				}

				return new Sheet( index );
			}
		}
	}

	public class Sheet : IDisposable
	{
		public Sheet( int index )
		{
			Index = index;
			Console.WriteLine( "Sheet " + Index + " Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "Sheet " + Index + " Dispose" );
		}

		public int Index { get; set; }

		public Range this[ int col, int row ]
		{
			get
			{
				if( col <= 0 || row <= 0 )
				{
					throw new ArgumentException();
				}

				return new Range( col, row );
			}
		}
	}

	public class Range : IDisposable
	{
		public Range( int col, int row )
		{
			Column = col;
			Row = row;
			Console.WriteLine( "Range (" + Column + ", " + Row + ") Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "Range (" + Column + ", " + Row + ") Dispose" );
		}

		public string Value
		{
			get { return "Range (" + Column + ", " + Row + ") Value: " + _value; }
			set { _value = value; }
		}

		public int Column { get; set; }
		public int Row { get; set; }

		private string _value = null;
	}
}