﻿using System;
using System.Collections.Generic;

namespace ExcelDummy
{
	class Disposer : IDisposable
	{
		public TYPE Push<TYPE>( TYPE disp_obj ) where TYPE : IDisposable
		{
			if( disp_obj != null )
			{
				_DispStack.Push( disp_obj );
			}
			return disp_obj;
		}

		public void Dispose()
		{
			foreach( IDisposable disp_obj in _DispStack )
			{
				disp_obj.Dispose();
			}
		}

		Stack<IDisposable> _DispStack = new Stack<IDisposable>();
	}

	public class ExcelApp : IDisposable
	{
		public ExcelApp()
		{
			Console.WriteLine( "ExcelApp Create" );
		}

		public void Dispose()
		{
			_Disposer.Dispose();

			Console.WriteLine( "ExcelApp Dispose" );
		}

		public BookCollection Books
		{
			get { return _Disposer.Push( new BookCollection() ); }
		}

		Disposer _Disposer = new Disposer();
	}

	public class BookCollection : IDisposable
	{
		public BookCollection()
		{
			Console.WriteLine( "BookCollection Create" );
		}

		public void Dispose()
		{
			_Disposer.Dispose();

			Console.WriteLine( "BookCollection Dispose" );
		}

		public Book Open( string book_path )
		{
			return _Disposer.Push( new Book( book_path ) );
		}

		Disposer _Disposer = new Disposer();
	}

	public class Book : IDisposable
	{
		public Book( string book_path )
		{
			if( string.IsNullOrEmpty( book_path ) )
			{
				throw new ArgumentNullException();
			}

			Path = book_path;
			Console.WriteLine( "Book(" + Path + ") Create" );
		}

		public void Dispose()
		{
			_Disposer.Dispose();

			Console.WriteLine( "Book(" + Path + ") Dispose" );
		}

		public SheetCollection Sheets
		{
			get { return _Disposer.Push( new SheetCollection() ); }
		}

		public string Path { get; set; }

		Disposer _Disposer = new Disposer();
	}

	public class SheetCollection : IDisposable
	{
		public SheetCollection()
		{
			Console.WriteLine( "SheetCollection Create" );
		}

		public void Dispose()
		{
			_Disposer.Dispose();

			Console.WriteLine( "SheetCollection Dispose" );
		}

		public Sheet this[ int index ]
		{
			get
			{
				if( index <= 0 )
				{
					throw new ArgumentException();
				}

				return _Disposer.Push( new Sheet( index ) );
			}
		}

		Disposer _Disposer = new Disposer();
	}

	public class Sheet : IDisposable
	{
		public Sheet( int index )
		{
			Index = index;
			Console.WriteLine( "Sheet " + Index + " Create" );
		}

		public void Dispose()
		{
			_Disposer.Dispose();

			Console.WriteLine( "Sheet " + Index + " Dispose" );
		}

		public int Index { get; set; }

		public Range this[ int col, int row ]
		{
			get
			{
				if( col <= 0 || row <= 0 )
				{
					throw new ArgumentException();
				}

				return _Disposer.Push( new Range( col, row ) );
			}
		}

		private Disposer _Disposer = new Disposer();
	}

	public class Range : IDisposable
	{
		public Range( int col, int row )
		{
			Column = col;
			Row = row;
			Console.WriteLine( "Range (" + Column + ", " + Row + ") Create" );
		}

		public void Dispose()
		{
			Console.WriteLine( "Range (" + Column + ", " + Row + ") Dispose" );
		}

		public string Value
		{
			get { return "Range (" + Column + ", " + Row + ") Value: " + _value; }
			set { _value = value; }
		}

		public int Column { get; set; }
		public int Row { get; set; }

		private string _value = null;
	}
}